<?php
/**
 * Implements some nice checks for REST modules
 */
class RESTcheck {
	/**
	 * Performs a preg_match, throwing an exception if the subject does not match
	 * @param string $pattern The pattern to check for
	 * @param mixed $subject The string to check
	 * @param string $name The name of the subject (parameter name or other)
	 * @param bool $throw Throw an exception if there is no match
	 * @throws RegExpRESTException
	 * @return bool
	 */
	function preg_match($pattern,$subject,$name='',$throw=true) {
		if(preg_match($pattern,$subject)) return true;
		if($throw) throw new RegExpRESTException($name, $pattern);
		return false;
	}
	/**
	 * Checks if a named parameter exists in this request
	 * @param string $name The name of the parameter to check
	 * @param bool $throw Throw an exception if there is no match
	 * @throws MissingParameterRESTException
	 * @return bool
	 */
	function parameter_exists($name,$throw=true) {
		$inputdata=REST::input()->getData();
		if(isset($inputdata[$name])) return true;
		if($throw) throw new MissingParameterRESTException($name);
		return false;
	}
	/**
	 * Checks if something is true
	 * @param bool $test Test wether this is true
	 * @param Exception $throw Throw this object if it is not true
	 * @throws Exception
	 * @return bool
	 */
	function assure($test,Exception &$throw=NULL) {
		if($test) return true;
		if($throw instanceof Exception) throw $throw;
		unset($throw);
		return false;
	}
}
