<?php
/**
 * The class containing input given to the application
 * @author Lars
 *
 */
class RESTinput {
	private $query='';
	private $module='';
	private $part='';
	private $data=array();
	/**
	 * Class constructor
	 * @param string $request_url
	 * @param int $method
	 * @throws MethodNotAllowedException
	 */
	function __construct($request_url,$method) {
		$pq=explode('?',$request_url,2);
		if(count($pq)==2) {
			list($part,$querystr)=$pq;
		}
		else {
			list($part)=$pq;
			$querystr='';
		}
		$md=explode('/', $part,2);
		if(count($md)==2) {
			list($this->module,$this->part)=$md;
		}
		else {
			list($this->module)=$md;
			$this->part='';
		}
		$this->query=$querystr;
		switch ($method) {
			case REST::get:
				parse_str($querystr,$this->data);
			break;
			case REST::post:
				$this->data=$_POST;
			break;
			case REST::put:
				parse_str(file_get_contents('php://input'),$this->data);
			break;
			case REST::delete:
				$this->data=false;
			break;
			default:
				throw new MethodNotAllowedException($method.' is not a valid method');
			break;
		}
	}
	/**
	 * Gets the module name
	 * @return string
	 */
	function getModule() {
		return $this->module;
	}
	/**
	 * Gets the action name
	 * @return string
	 */
	function getPart() {
		return $this->part;
	}
	/**
	 * Gets the action name separated by / in an array if no id is given.
	 * Gets the action name at position $id if an id is given.
	 * @param int $id The position of the action name
	 * @return mixed Array if no id is given, string if an id is given and exists, false if an id is given and does not exist
	 */
	function getParts($id=NULL) {
		$parts=explode('/',$this->part);
		if(($last=array_pop($parts))!='') array_push($parts, $last);
		if($id===NULL) return $parts;
		if(array_key_exists($id, $parts)) return $parts[$id];
		return false;
	}
	/**
	 * Gets the full querystring
	 * @return string
	 */
	function getQueryString() {
		return $this->query;
	}
	/**
	 * Gets the data associated with the request
	 * @return array
	 */
	function getData() {
		return $this->data;
	}
	/**
	 * Decodes input data according to the data type
	 * @param string $data The data to decode
	 * @return mixed The decoded data
	 * @throws UnexpectedValueException
	 */
	function decode($data) {
		switch(REST::getDataType()) {
			case REST::json:
				return json_decode($data,true);
				break;
			case REST::serialize:
				return unserialize($data);
				break;
			default:
				throw new UnexpectedValueException('REST data type should be json or serialize');
		}
	}
}