<?php
/**
 * Class to write the response
 * @author Lars
 *
 */
class RESTresponse {
	private $output;
	private $dataType;
	private $statuscode=200;
	/**
	 * Class constructor
	 * @param int $dataType
	 * @param RESToutput $output
	 * @param int $statuscode
	 */
	function __construct($dataType,RESToutput $output,$statuscode=200) {
		$this->output=$output->getData();
		if(isset($this->output['status'])&&$this->output['status']===false) $statuscode=500; //Our status is not good, 500 error
		$this->dataType=$dataType;
		$this->statuscode=$statuscode;
	}
	/**
	 * Sens the output to the client
	 */
	function send() {
		header('HTTP/1.1 '.$this->statuscode.' '.$this->mapStatusCode($this->statuscode));
		if($this->dataType==REST::json) {
			echo json_encode($this->output);
		}
		elseif($this->dataType==REST::jsonp) {
			echo $_GET['callback'].'('.json_encode($this->output).');';
		}
		elseif($this->dataType==REST::serialize) {
			echo serialize($this->output);
		}
	}
	/**
	 * Maps numeric status codes to status strings
	 * @param int $status The HTTP status code
	 * @return string The string corresponding to the status code
	 */
	private function mapStatusCode($status) {
		$codes = Array(
		    100 => 'Continue',
		    101 => 'Switching Protocols',
		    200 => 'OK',
		    201 => 'Created',
		    202 => 'Accepted',
		    203 => 'Non-Authoritative Information',
		    204 => 'No Content',
		    205 => 'Reset Content',
		    206 => 'Partial Content',
		    300 => 'Multiple Choices',
		    301 => 'Moved Permanently',
		    302 => 'Found',
		    303 => 'See Other',
		    304 => 'Not Modified',
		    305 => 'Use Proxy',
		    306 => '(Unused)',
		    307 => 'Temporary Redirect',
		    400 => 'Bad Request',
		    401 => 'Unauthorized',
		    402 => 'Payment Required',
		    403 => 'Forbidden',
		    404 => 'Not Found',
		    405 => 'Method Not Allowed',
		    406 => 'Not Acceptable',
		    407 => 'Proxy Authentication Required',
		    408 => 'Request Timeout',
		    409 => 'Conflict',
		    410 => 'Gone',
		    411 => 'Length Required',
		    412 => 'Precondition Failed',
		    413 => 'Request Entity Too Large',
		    414 => 'Request-URI Too Long',
		    415 => 'Unsupported Media Type',
		    416 => 'Requested Range Not Satisfiable',
		    417 => 'Expectation Failed',
		    500 => 'Internal Server Error',
		    501 => 'Not Implemented',
		    502 => 'Bad Gateway',
		    503 => 'Service Unavailable',
		    504 => 'Gateway Timeout',
		    505 => 'HTTP Version Not Supported'
		);

		return (isset($codes[$status])) ? $codes[$status] : '';
	}
}