<?php
$wd=getcwd();
chdir(__DIR__.'/../..');
require 'inc/controllers.php';
chdir($wd);
class auth {
	static $consumer_id=0;
	const read_lang=1;
	const write_lang=2;
	const read_mail_addr=4;
	const read_mail=8;
	const write_mail=16;
	const read_db=32;
	const write_db_groups=64;
	const write_db_sites_new=128;
	const write_db_sites_edit=256;
	const write_db_change_admins=512;
	const write_db_users=1024;
	const all=2048;
	static private $sessid=NULL;
	static function init() {
		if(!isset($_COOKIE['APISESSID'])) throw new PreconditionFailedException('We need an APISESSID cookie');
		self::$sessid=$_COOKIE['APISESSID'];
		$session=self::get_granted_data(self::$sessid);
		if(isset($session['consumer'])) {
			self::$consumer_id=$session['consumer'];
		}
		if(isset($session['user'])&&self::auth_granted($session['user'],self::$consumer_id)) {
			session::api_init($session['user']);
		}
		else {
			throw new UnauthorizedException('User did not grant permission to access any resource, or authentication expired.');
		}
	}
	/**
	 * Checks if the user has granted access to the consumer
	 * @param string $guid The guid of the resource owner
	 * @param string $consumer The consumer id
	 */
	static private function auth_granted($guid,$consumer) {
		global $local_connection;
		return ($local_connection->query('SELECT * FROM `auth_granted` WHERE `user`=\''.$guid.'\' AND `consumer`='.$consumer.' AND (`until`>CURRENT_TIMESTAMP OR `until` IS NULL)')->num_rows>0);
	}
	/**
	 * Creates a temporary key for authentication
	 * @param int $consumer The consumer id
	 * @param string $return_url The url to return to after authentication
	 * @param int $use Specify valid use for the authentication
	 * @return string Temporary key
	 */
	static private function generate_temporary($consumer,$return_url,$use){
		global $local_connection;
		$local_connection->query('INSERT INTO `auth_pending` VALUES(NULL,'.intval($consumer).','.intval($use).', SHA1(RAND()),\''.$local_connection->escape_string($return_url).'\',NULL)');
		$id=$local_connection->insert_id;
		return $local_connection->query('SELECT * FROM `auth_pending` WHERE `id`='.$id)->fetch_object()->tempkey;
	}
	/**
	 * Checks if the API key is valid for this host
	 * @param string $key
	 * @return bool
	 */
	static private function check_api_key($key) {
		global $local_connection;
		return ($local_connection->query('SELECT * FROM `auth_consumers` WHERE `shared_secret`=\''.$local_connection->escape_string($key).'\' AND `ip`='.ip2long($_SERVER['REMOTE_ADDR']).'')->num_rows!=0);
	}
	/**
	 * Gets information of the temporary credentials
	 * @param string $tempcred The temporary credentials
	 * @return bool|array False when the temporary credentials are no longer valid, Array with credential information if valid
	 */
	static function get_temporary($tempcred) {
		global $local_connection;
		$q=$local_connection->query('SELECT * FROM `auth_pending` WHERE `tempkey`=\''.$local_connection->escape_string($tempcred).'\'');
		if($q->num_rows==0) return false;
		return $q->fetch_assoc();
	}
	/**
	 * Initializes the authentication process
	 * @param string $api_key
	 * @param string $return_url
	 * @throws ForbiddenException
	 * @return string The temporary credentials
	 */
	static function auth_initialize($api_key,$return_url,$use) {
		global $local_connection;
		if(!self::check_api_key($api_key)) {
			throw new ForbiddenException('Invalid API key given.');
		}
		$consumerq=$local_connection->query('SELECT * FROM `auth_consumers` WHERE `shared_secret`=\''.$local_connection->escape_string($api_key).'\'');
		$consumerinfo=$consumerq->fetch_assoc();
		self::$consumer_id=$consumerinfo['id'];
		$_SESSION['consumer_id']=self::$consumer_id;
		$tempcred=self::generate_temporary(self::$consumer_id, $return_url,$use);
		return $tempcred;
	}
	/**
	 * Authorizes the consumer to use the specified resource
	 * @param string $userid User GUID
	 * @param string $tempcred Consumer temporary credentials
	 * @param int $time How long may the application use the resources
	 * @throws NotFoundException
	 * @throws GoneException
	 * @return string The credentials to access API resources
	 */
	static function auth_authorize($userid,$tempcred,$time=NULL) {
		if(!session::userExists($userid)) {
			throw new NotFoundException('The user '.$userid.' does not exist');
		}
		$credinfo=self::get_temporary($tempcred);
		if($credinfo===false) {
			throw new GoneException('These temporary credentials are no longer valid');
		}
		global $local_connection;
		// Invalidate the temporary credentials
		$local_connection->query('DELETE FROM `auth_pending` WHERE `tempkey`=\''.$local_connection->escape_string($tempcred).'\'');
		if($time!==NULL) {
			$time_allowed='FROM_UNIXTIME(UNIX_TIMESTAMP(NOW())+'.$local_connection->escape_string($time).')';
		}
		else  {
			$time_allowed='NULL';
		}
		$local_connection->query('INSERT INTO `auth_granted` VALUES(NULL,\''.$userid.'\','.$credinfo['consumer'].','.$credinfo['use'].',\''.sha1(rand()).'\','.$time_allowed.')');
		return $local_connection->query('SELECT * FROM `auth_granted` WHERE `id`='.$local_connection->insert_id)->fetch_object()->identifier;
	}
	/**
	 * Rejects the authentication attempt
	 * @param string $tempcred Consumer temporary credentials
	 * @return bool
	 */
	static function auth_reject($tempcred) {
		global $local_connection;
		return $local_connection->query('DELETE FROM `auth_pending` WHERE `tempkey`=\''.$local_connection->escape_string($tempcred).'\'');
	}
	/**
	 * Get infomation about a consumer
	 * @param int $consumer_id The consumer id
	 * @return bool|array False if the consumer does not exist, array with consumerinformation  if it does exist;
	 */
	static function get_consumer_info($consumer_id) {
		global $local_connection;
		$consumerq=$local_connection->query('SELECT * FROM `auth_consumers` WHERE `id`='.$local_connection->escape_string($consumer_id));
		if($consumerq->num_rows!=1) return false;
		return $consumerq->fetch_assoc();
	}
	/**
	 * Sets the consumer id from the temporary credentials
	 * @param string $tempcred The temporary credentials
	 */
	static function set_consumer($tempcred) {
		$tcredinfo=self::get_temporary($tempcred);
		if($tcredinfo!==false) self::$consumer_id=$tcredinfo['consumer'];
		else throw new GoneException('These temporary credentials are no longer valid');
	}
	/**
	 * Gets all access permissions for the application
	 * @param int $aci The access identifier
	 * @param bool $not List what the application can't do instead
	 * @return array Array containing translated permissions
	 */
	static function access_list($aci,$not=false) {
		global $t;
		$aci=(int)$aci;
		if($not) $aci=~$aci;
		$return=array();
		if(!$not) $return[]=$t->_('Verify this account is yours');
		if($aci&1) $return[]=$t->_('Read your language preferences');
		if($aci&2) $return[]=$t->_('Edit your language preferences');
		if($aci&4) $return[]=$t->_('Read your e-mail address');
		if($aci&8) $return[]=$t->_('Read your messages');
		if($aci&16) $return[]=$t->_('Send and remove messages');
		if($aci&32) $return[]=$t->_('Read all data in your databases');
		if($aci&64) $return[]=$t->_('Add users to groups in your databases');
		if($aci&128) $return[]=$t->_('Create and delete sites in your databases');
		if($aci&256) $return[]=$t->_('Edit sites in your databases');
		if($aci&512) $return[]=$t->_('Change site administrators and group administrators in your databases');
		if($aci&1024) $return[]=$t->_('Add users to your databases');
		if($aci&2048) $return[]=$t->_('Do <b style="color: red">everything</b> else that is possible');
		if($not) $return[]=$t->_('See your password');
		return $return;
	}
	/**
	 * Gets information about the identifier (credentials)
	 * @param string $identifier
	 * @return array
	 */
	static private function get_granted_data($identifier) {
		global $local_connection;
		$q=$local_connection->query('SELECT * FROM `auth_granted` WHERE `identifier`=\''.$local_connection->escape_string($identifier).'\'');
		if($q->num_rows==0) return array();
		return $q->fetch_assoc();
	}
	/**
	 * Checks wether a permission is granted
	 * @param int $aci The access control identifier to check
	 * @param bool $throw_on_failure Throw unauthorizedException on failure if true
	 * @throws UnauthorizedException
	 * @return bool true if permission is granted, false if not (or exception)
	 */
	static function granted($aci,$throw_on_failure=false) {
		global $local_connection;
		$access=(int)$local_connection->query('SELECT * FROM `auth_granted` WHERE `identifier`=\''.self::$sessid.'\'')->fetch_object()->use;
		if($access&$aci) return true;
		elseif($throw_on_failure) throw new UnauthorizedException('You don\'t have the right permissions');
		return false;
	}

}
