<?php
abstract class fs {
	/**
	 * Converts bytes to human-readable file sizes
	 * @param int $size The size in bytes to convert
	 * @return string
	 */
	final function sizetohuman($size) {
		$sizes = array(" Bytes", " KB", " MB", " GB", " TB", " PB", " EB", " ZB", " YB");
		if ($size == 0) { return('n/a'); } else {
		return (round($size/pow(1024, ($i = floor(log($size, 1024)))), 2) . $sizes[$i]);
		}
	}
	/**
	 * Removes /./ and resolves /../
	 * @param string $path
	 * @return string The new path
	 * @internal Returns NULL when an invalid path is given
	 */
	final function simplify_path($path) {
		if($path=='.'||$path=='\\'||$path=='') $path='/';
		if(!preg_match('@^([^\0])+$@',$path)) {  return NULL; }
		$path = str_replace(array('/', '\\'), '/', $path);
		$parts = array_filter(explode('/', $path), 'strlen');
		$absolutes = array();
		foreach ($parts as $part) {
			if ('.' == $part) continue;
			if ('..' == $part) {
				array_pop($absolutes);
			} else {
				$absolutes[] = $part;
			}
		}
        $ret=implode('/', $absolutes);
        if($ret=='') return '/';
        return $ret;
	}
	/**
	 * Raise a php error
	 * @param string $error The errorstring
	 * @param int $level The errorlevel
	 * @internal Remove the trigger_error line to make MySQLfs silent
	 */
	final protected function trigger_error($error,$level) {
		trigger_error($error,$level);
	}
	/**
	 * Gets the content type based on the extention
	 * @param string $path
	 * @return string mimetype of the file
	 */
	final function mime_content_type($path) {
		if(($path=$this->simplify_path($path))===NULL)  { $this->trigger_error("MySQLfs: ".func_get_arg(0)." is an invalid pathname",E_USER_WARNING); return false; }
		$mime_types = array("323" => "text/h323","acx" => "application/internet-property-stream","ai" => "application/postscript","aif" => "audio/x-aiff","aifc" => "audio/x-aiff","aiff" => "audio/x-aiff","asf" => "video/x-ms-asf","asr" => "video/x-ms-asf","asx" => "video/x-ms-asf","au" => "audio/basic","avi" => "video/x-msvideo","axs" => "application/olescript","bas" => "text/plain","bcpio" => "application/x-bcpio","bin" => "application/octet-stream","bmp" => "image/bmp","c" => "text/plain","cat" => "application/vnd.ms-pkiseccat","cdf" => "application/x-cdf","cer" => "application/x-x509-ca-cert","class" => "application/octet-stream","clp" => "application/x-msclip","cmx" => "image/x-cmx","cod" => "image/cis-cod","cpio" => "application/x-cpio","crd" => "application/x-mscardfile","crl" => "application/pkix-crl","crt" => "application/x-x509-ca-cert","csh" => "application/x-csh","css" => "text/css","dcr" => "application/x-director","der" => "application/x-x509-ca-cert","dir" => "application/x-director","dll" => "application/x-msdownload","dms" => "application/octet-stream","doc" => "application/msword","dot" => "application/msword","dvi" => "application/x-dvi","dxr" => "application/x-director","eps" => "application/postscript","etx" => "text/x-setext","evy" => "application/envoy","exe" => "application/octet-stream","fif" => "application/fractals","flr" => "x-world/x-vrml","gif" => "image/gif","gtar" => "application/x-gtar","gz" => "application/x-gzip","h" => "text/plain","hdf" => "application/x-hdf","hlp" => "application/winhlp","hqx" => "application/mac-binhex40","hta" => "application/hta","htc" => "text/x-component","htm" => "text/html","html" => "text/html","htt" => "text/webviewhtml","ico" => "image/x-icon","ief" => "image/ief","iii" => "application/x-iphone","ins" => "application/x-internet-signup","isp" => "application/x-internet-signup","jfif" => "image/pipeg","jpe" => "image/jpeg","jpeg" => "image/jpeg","jpg" => "image/jpeg","js" => "application/x-javascript","latex" => "application/x-latex","lha" => "application/octet-stream","lsf" => "video/x-la-asf","lsx" => "video/x-la-asf","lzh" => "application/octet-stream","m13" => "application/x-msmediaview","m14" => "application/x-msmediaview","m3u" => "audio/x-mpegurl","man" => "application/x-troff-man","mdb" => "application/x-msaccess","me" => "application/x-troff-me","mht" => "message/rfc822","mhtml" => "message/rfc822","mid" => "audio/mid","mny" => "application/x-msmoney","mov" => "video/quicktime","movie" => "video/x-sgi-movie","mp2" => "video/mpeg","mp3" => "audio/mpeg","mpa" => "video/mpeg","mpe" => "video/mpeg","mpeg" => "video/mpeg","mpg" => "video/mpeg","mpp" => "application/vnd.ms-project","mpv2" => "video/mpeg","ms" => "application/x-troff-ms","mvb" => "application/x-msmediaview","nws" => "message/rfc822","oda" => "application/oda","p10" => "application/pkcs10","p12" => "application/x-pkcs12","p7b" => "application/x-pkcs7-certificates","p7c" => "application/x-pkcs7-mime","p7m" => "application/x-pkcs7-mime","p7r" => "application/x-pkcs7-certreqresp","p7s" => "application/x-pkcs7-signature","pbm" => "image/x-portable-bitmap","pdf" => "application/pdf","pfx" => "application/x-pkcs12","pgm" => "image/x-portable-graymap","pko" => "application/ynd.ms-pkipko","pma" => "application/x-perfmon","pmc" => "application/x-perfmon","pml" => "application/x-perfmon","pmr" => "application/x-perfmon","pmw" => "application/x-perfmon","pnm" => "image/x-portable-anymap","pot" => "application/vnd.ms-powerpoint","ppm" => "image/x-portable-pixmap","pps" => "application/vnd.ms-powerpoint","ppt" => "application/vnd.ms-powerpoint","prf" => "application/pics-rules","ps" => "application/postscript","pub" => "application/x-mspublisher","qt" => "video/quicktime","ra" => "audio/x-pn-realaudio","ram" => "audio/x-pn-realaudio","ras" => "image/x-cmu-raster","rgb" => "image/x-rgb","rmi" => "audio/mid","roff" => "application/x-troff","rtf" => "application/rtf","rtx" => "text/richtext","scd" => "application/x-msschedule","sct" => "text/scriptlet","setpay" => "application/set-payment-initiation","setreg" => "application/set-registration-initiation","sh" => "application/x-sh","shar" => "application/x-shar","sit" => "application/x-stuffit","snd" => "audio/basic","spc" => "application/x-pkcs7-certificates","spl" => "application/futuresplash","src" => "application/x-wais-source","sst" => "application/vnd.ms-pkicertstore","stl" => "application/vnd.ms-pkistl","stm" => "text/html","svg" => "image/svg+xml","sv4cpio" => "application/x-sv4cpio","sv4crc" => "application/x-sv4crc","t" => "application/x-troff","tar" => "application/x-tar","tcl" => "application/x-tcl","tex" => "application/x-tex","texi" => "application/x-texinfo","texinfo" => "application/x-texinfo","tgz" => "application/x-compressed","tif" => "image/tiff","tiff" => "image/tiff","tr" => "application/x-troff","trm" => "application/x-msterminal","tsv" => "text/tab-separated-values","txt" => "text/plain","uls" => "text/iuls","ustar" => "application/x-ustar","vcf" => "text/x-vcard","vrml" => "x-world/x-vrml","wav" => "audio/x-wav","wcm" => "application/vnd.ms-works","wdb" => "application/vnd.ms-works","wks" => "application/vnd.ms-works","wmf" => "application/x-msmetafile","wps" => "application/vnd.ms-works","wri" => "application/x-mswrite","wrl" => "x-world/x-vrml","wrz" => "x-world/x-vrml","xaf" => "x-world/x-vrml","xbm" => "image/x-xbitmap","xla" => "application/vnd.ms-excel","xlc" => "application/vnd.ms-excel","xlm" => "application/vnd.ms-excel","xls" => "application/vnd.ms-excel","xlt" => "application/vnd.ms-excel","xlw" => "application/vnd.ms-excel","xof" => "x-world/x-vrml","xpm" => "image/x-xpixmap","xwd" => "image/x-xwindowdump","z" => "application/x-compress","zip" => "application/zip");
		$filename=basename($path);
		$parts=explode('.',$filename);
		$ext=$parts[count($parts)-1];
		if(!array_key_exists($ext,$mime_types))  return 'application/octet-stream'; //Not in array? Must be default!
		return $mime_types[$ext];
	}
	final function dirname($path) {
		return $this->simplify_path($path.'/..');
	}
	final function basename($path) {
		return basename($path);
	}
	/**
	 * Reads the directory into an array
	 * @param string $path The path to read
	 * @return array Array with nested arrays containing information about the file
	 */
	abstract function readdir_full($path);
	/**
	 * Gets the file group.
	 * @param string $path
	 * @return string
	 */
	abstract function filegroup($path);
	/**
	 * This function returns the time when the data blocks of a file were being written to,
	 * that is, the time when the content of the file was changed.
	 * @param string $path
	 * @return int UNIX timestamp
	 */
	abstract function filemtime($path);
	/**
	 * Gets the file owner.
	 * @param string $path
	 * @return string
	 */
	abstract function fileowner($path);
	/**
	 * Gets permissions for the given file.
	 * @param string $path
	 * @return string The octal rights on the file
	 */
	abstract function fileperms($path);
	/**
	 * Gets the size for the given file in bytes
	 * @param string $path
	 * @return int the filesize in bytes
	 */
	abstract function filesize($path);
	/**
	 * Returns the type of the given file.
	 * @param string $path
	 * @return string 'file' if the path is a file, 'directory' if the path is a directory
	 */
	abstract function filetype($path);
	/**
	 * Checks wether a path is a file or not.
	 * @param string $path
	 * @return bool true if the path is a file, false if not.
	 */
	abstract function is_file($path);
	/**
	 * Checks wether a path if a directory or not.
	 * @param string $path
	 * @return bool true if the path is a directory, false if not.
	 */
	abstract function is_dir($path);
	/**
	 * Returns contents of the file at $path or false if $path isn't a file
	 * @param string $path
	 * @return mixed The file's contents if it is a file, false if it isn't a file.
	 */
	abstract function file_get_contents($path);
	/**
	 * Reads an entire file into an array.
	 * @param string $path
	 * @param int $flags
	 * @return array
	 */
	abstract function file($path,$flags);
	/**
	 * Checks if a directory is empty
	 * @param string $path
	 * @return bool true if $path is empty, false if the directory is not empty
	 * @internal returns true when a file is given
	 */
	abstract function is_empty($path);
	/**
	 * Removes a directory
	 * @param string $path
	 * @return Returns true on success and false on failure
	 */
	abstract function rmdir($path);
	/**
	 * Removes a file
	 * @param string $path
	 * @return bool true on success, false on failure
	 */
	abstract function unlink($path);
	/**
	 * Checks wether a file exists
	 * @param string $path
	 * @return bool true if $path exists, false if $path doesn't exist
	 */
	abstract function file_exists($path);
	/**
	 * Creates a directory
	 * @param string $path
	 * @param int $mode Octal permissions of the file. If not defined, it takes the permissions of the parent directory
	 * @return bool
	 */
	abstract function mkdir($path,$mode);
	/**
	 * Creates a file
	 * @param string $path
	 * @param int $mode Octal permissions of the file. If not defined, it takes the permissions of the parent directory
	 * @param string $mime The mimetype of the file, if not defined, the mimetype will be looked up by extention
	 * @return bool
	 */
	abstract function mkfile($path,$mode,$mime);
	/**
	 * Writes data to the file
	 * @param string $path The path to write to
	 * @param string $data The data to write to the file
	 * @return bool
	 */
	abstract function file_put_contents($path,$data);
	/**
	 * Checks wether a file is readable
	 * @param string $path
	 * @return bool true is the file is readable, false otherwise
	 */
	abstract function is_readable($path);
	/**
	 * Checks if the file is writeable
	 * @param string $path
	 * @return bool true if the file is writeable, false otherwise
	 */
	abstract function is_writable($path);
	/**
	 * Alias of is_writable
	 * @param string $path
	 * @ignore
	 */
	final function is_writeable($path) {
		return $this->is_writable($path);
	}
	/**
	 * Checks if the file is executable by php
	 * @param string $path
	 * @return bool true if the file is executable, false otherwise
	 */
	abstract function is_executable($path);
	/**
	 * Change permissions
	 * @param string $path
	 * @param int $mode The new permissions for the file
	 * @return bool
	 */
	abstract function chmod($path,$mode);
	/**
	 * Change owner
	 * @param string $path
	 * @param string $user New user of the file
	 * @uses mysqlfs_validate::valid_user_function()
	 * @return bool
	 */
	abstract function chown($path,$user);
	/**
	 * Changes group
	 * @param string $path
	 * @param string $group New group of the file
	 * @uses mysqlfs_validate::valid_group_function()
	 * @return bool
	 */
	abstract function chgrp($path,$group);
	/**
	 * Rename a file
	 * @param string $oldpath The current location of the file
	 * @param string $newpath The new location of the file
	 * @return bool
	 * @internal You must have reading and writing permissions on $oldpath and writing permissions on $newpath
	 */
	abstract function rename($oldpath,$newpath);
	/**
	 * Copy a file to another location
	 * @param string $source The current location of the file
	 * @param string $dest The location you want the file to be copied to
	 * @internal You must have reading permissions on $source and writing permissions on $dest
	 * @return bool
	 */
	abstract function copy($source,$dest);
	/**
	 * Moves a newly uploaded file to its final location
	 * @param string $source The source path on the real filesystem
	 * @param string $dest The destiniation on MySQLfs
	 * @internal This function checks to ensure that the file designated by filename is a valid upload file (meaning that it was uploaded via PHP's HTTP POST upload mechanism).
	 * @internal If the file is valid, it will be moved to the filename given by destination.
	 * @return bool
	 */
	abstract function move_uploaded_file($source,$dest);
	/**
	 * Reads a file and writes it to the output buffer
	 * @param string $path
	 * @return int The length of the written string
	 */
	abstract function readfile($path);
	/**
	 * Returns an array of files and directories from the directory.
	 * @param string $path
	 * @param int $sorting_order sort ascending if omitted or 0, descending if 1
	 * @return array Array containing the filenames
	 */
	abstract function scandir($path,$sorting_order);
	
}