<?php
/**
 * Handles remote database groups
 * @since 2.0
 * @package remote
 */
class groups {
	/**
	 * Holds the groups the user is member of
	 * @var array
	 */
	private static $groups=array();
	/**
	 * Holds the MySQLi connection
	 * @var mysqli
	 */
	private static $dbconn=NULL;
	/**
	 * Initializes the static group class
	 * @uses session::$guid
	 */
	static function init(){
		global $remote_connection;
		self::$dbconn=$remote_connection;
		$q=self::$dbconn->query('SELECT * FROM `groups` WHERE `user`=\''.self::$dbconn->escape_string(session::$guid).'\'');
		while($group=$q->fetch_assoc()) {
			self::$groups[]=$group['gid'];
		}
	}
	/**
	 * Adds a user to a group
	 * @param string $user The user you want to add
	 * @param string $group The group you want to add the user to
	 * @return bool
	 * @uses JSError::add()
	 * @uses session::userExists()
	 */
	static function addUser($user,$group) {
		if(!session::userExists($user)) {
			JSError::add($t->_('User does not exist'));
			return false;
		}
		if(!self::groupExists($group)) {
			JSError::add($t->_('Group does not exist'));
			return false;
		}
		if(!self::isMyGroup($group)) {
			JSError::add($t->_('You cannot add someone to a group you don\'t belong to'));
			return false;
		}
		if(!self::isGroupAdmin($group)) {
			JSError::add($t->_('You are not an administrator of that group'));
			return false;
		}
		return self::$dbconn->query('INSERT INTO `groups` VALUES(\''.self::$dbconn->escape_string($group).'\',\''.self::$dbconn->escape_string($user).'\',0)');
	}
	/**
	 * Checks wether a group exists
	 * @param string $group
	 * @param int $by Get by GUID if 0 or omitted, get by groupname if 1
	 * @return bool
	 */
	static function groupExists($group,$by=0) {
		if($group==NULL) return true;
		if($by==0)
			$q=self::$dbconn->query('SELECT * FROM `groups_names` WHERE `gid`=\''.self::$dbconn->escape_string($group).'\'');
		elseif($by==1)
			$q=self::$dbconn->query('SELECT * FROM `groups_names` WHERE `name`=\''.self::$dbconn->escape_string($group).'\'');
		else {
			trigger_error('Unexpected value of argument 2',E_USER_WARNING); //We do not support your value.
			return;
		}
		return ($q->num_rows==1);
	}
	/**
	 * Checks if you are in the group
	 * @param string $group
	 * @return bool
	 */
	static function isMyGroup($group) {
		if($group==NULL) return true;
		return in_array($group, self::$groups);
	}
	/**
	 * Checks if you are a group admin
	 * @param string $group
	 * @return bool
	 */
	static function isGroupAdmin($group) {
		if(!self::isMyGroup($group)) return false;
		$a=self::$dbconn->query('SELECT * FROM `groups` WHERE `gid`=\''.self::$dbconn->escape_string($group).'\' AND `user`=\''.self::$dbconn->escape_string(session::$guid).'\'')->fetch_assoc();
		return $a['admin']==1;
	}
	/**
	 * Gets the group's name
	 * @param $group
	 * @return string The group's canonical name
	 */
	static function getName($group) {
		if($group==NULL) return '';
		$a=self::$dbconn->query('SELECT * FROM `groups_names` WHERE `gid`=\''.self::$dbconn->escape_string($group).'\'')->fetch_assoc();
		return $a['name'];
	}
	/**
	 * Gets the group GUID from the group name
	 * @param string $group The group's name
	 * @return string the group's GUID
	 */
	static function getGUID($group) {
		if($group==NULL) return '';
		$a=self::$dbconn->query('SELECT * FROM `groups_names` WHERE `name`=\''.self::$dbconn->escape_string($group).'\'')->fetch_assoc();
		return $a['gid'];
	}
	/**
	 * Get all your groups
	 * @return array All groups by GUID in an array
	 */
	static function getMyGroups() {
			return self::$groups;
	}

}
groups::init();