<?php
/**
 * RemoteCP internal mail class
 */
class mail {
	/**
	 * Holds the database connection
	 * @var MySQLi
	 */
	private $database=NULL;
	/**
	 * Holds the user identifier
	 * @var string
	 */
	private $user='';
	/**
	 * Initializes the mail class
	 * @param MySQLi $db The database connection
	 * @param string $user The user identifier of the user to read mail
	 */
	function __construct($db,$user) {
		$this->database=$db;
		$this->user=$user;
	}
	/**
	 * Gets the inbox of the current user
	 * @return array An array of message id's
	 */
	function getInbox() {
		$q=$this->database->query('SELECT `mid` FROM `messages_receivers` WHERE `receiver`=\''.$this->user.'\' AND `del`=0 AND `pdel`=0');
		$messages=array();
		while($msg=$q->fetch_assoc()) {
			$messages[]=$msg['mid'];
		}
		$q->free();
		return $messages;
	}
	/**
	 * Gets the deleted messages of the current user
	 * @return array An array of message id's
	 */
	function getDeleted() {
		$q=$this->database->query('SELECT `mid` FROM `messages_receivers` WHERE `receiver`=\''.$this->user.'\' AND `del`=1 AND `pdel`=0');
		$messages=array();
		while($msg=$q->fetch_assoc()) {
			$messages[]=$msg['mid'];
		}
		$q->free();
		return $messages;
	}
	/**
	 * Gets all sent messages of the current user
	 * @return array An array of message id's
	 */
	function getSent() {
		$q=$this->database->query('SELECT * FROM `messages_main` WHERE `sender`=\''.$this->user.'\'');
		$messages=array();
		while($msg=$q->fetch_assoc()) {
			$messages[]=$msg['mid'];
		}
		$q->free();
		return $messages;
	}
	/**
	 * Gets information about a message
	 * @param int $mid The id of the message to get
	 * @return array|bool An array with information about the message
	 */
	function getMessage($mid) {
		$q1=$this->database->query('SELECT * FROM `messages_main` WHERE `mid`=\''.$this->database->escape_string($mid).'\'');
		$q2=$this->database->query('SELECT * FROM `messages_receivers` WHERE `mid`=\''.$this->database->escape_string($mid).'\' AND `receiver`=\''.$this->user.'\'');
		if(!$q1||!$q2||$q1->num_rows==0||$q2->num_rows==0) return false;
		$messageinfo=array_merge($q1->fetch_assoc(),$q2->fetch_assoc());
		$q1->free();
		$q2->free();
		unset($messageinfo['receiver']);
		return $messageinfo;	
	}
	/**
	 * Gets user id's of all users receiving a message
	 * @param int $mid The id of the message
	 * @return array The userids of the users receiving the message
	 */
	function getReceivers($mid) {
		$q=$this->database->query('SELECT `receiver` FROM `messages_receivers` WHERE `mid`=\''.$this->database->escape_string($mid).'\'');
		$recvs=array();
		while($recv=$q->fetch_assoc()) {
			$recvs[]=$recv['receiver'];
		}
		$q->free();
		return $recvs;
	}
	/**
	 * Sets a column in messages_receivers for the give message
	 * @param int $mid The id of the message
	 * @param string $col The column to set
	 * @param string $value The value to set the column to
	 * @return bool
	 */
	function setMessageInfo($mid,$col,$value) {
		return $this->database->query('UPDATE `messages_receivers` SET `'.$this->database->escape_string($col).'`=\''.$this->database->escape_string($value).'\' WHERE `mid`=\''.$this->database->escape_string($mid).'\' AND `receiver`=\''.$this->user.'\'');
	}
	/**
	 * Gets a column in messages_receivers for the given message
	 * @param int $mid The id of the message
	 * @param string $col The column to get
	 * @return mixed The value of the column
	 */
	function getMessageInfo($mid,$col) {
		$q=$this->database->query('SELECT * FROM `messages_receivers` WHERE `mid`=\''.$this->database->escape_string($mid).'\' AND `receiver`=\''.$this->user.'\'');
		if(!$q||$q->num_rows==0) return false;
		$a=$q->fetch_array();
		if(!array_key_exists($col, $a)) return false;
		return $a[$col];
	}
	/**
	 * Sends an message
	 * @param array $receivers The userids to send the message to
	 * @param string $subject The subject of the message
	 * @param string $message The message itself
	 * @return bool
	 */
	function send($receivers,$subject,$message) {
		$q=array();
		$q1=$this->database->query('INSERT INTO `messages_main` VALUES(NULL,\''.$this->user.'\', \''.$this->database->escape_string($subject).'\', \''.$this->database->escape_string($message).'\',NULL)');
		if(!$q1) return false;
		$mid=$this->database->insert_id;
		foreach($receivers as $receiver) {
			$q[]=$this->database->query('INSERT INTO `messages_receivers` VALUES('.$mid.',\''.$this->database->escape_string($receiver).'\',0,0,0)');
		}
		if(in_array(false, $q, true)) return false;
		return true;
	}
	/**
	 * Reads a message
	 * @param int $mid The id of the message
	 * @return array|bool An array with information about the message
	 */
	function read($mid) {
		$minfo=$this->getMessage($mid);
		if(!in_array($this->user,$this->getReceivers($mid))&&$this->user!=$minfo['sender']) return false;
		if($this->getMessageInfo($mid, 'read')==0) $this->setMessageInfo($mid, 'read', 1);
		$msginfo=$this->getMessage($mid);
		$msginfo['receivers']=$this->getReceivers($mid);
		return $msginfo;
	}
	/**
	 * Deletes a message
	 * @param int $mid The id of the message
	 * @return bool
	 * @internal User should be receiver to delete message
	 */
	function delete($mid) {
	//	var_dump($this->user,$this->getReceivers($mid));
		if(!in_array($this->user,$this->getReceivers($mid))) return false;
		if($this->getMessageInfo($mid, 'del')==0) return $this->setMessageInfo($mid, 'del', 1);
		else return $this->setMessageInfo($mid,'pdel',1);
	}
}