<?php
class tinymce {
	const tinymce=0;
	const jquery=1;
	const advanced=0;
	const simple=1;
	const defaultSelector=NULL;
	/**
	 * Gets the configuration parameters for a simple editor
	 * @param bool $save Turn the save plugin on or off
	 * @return string
	 */
	static private function getConfigSimple($save) {
		global $t;
		return '
			language: "'.$t->get_language().'",
			theme : "advanced",
			skin : "o2k7",
			skin_variant : "black",
			plugins : "safari,spellchecker,pagebreak,style,layer,table,save,tinyautosave,advhr,advimage,advlink,emotions,iespell,inlinepopups,insertdatetime,preview,media,searchreplace,print,contextmenu,paste,directionality,fullscreen,noneditable,visualchars,nonbreaking,xhtmlxtras,template",
			theme_advanced_buttons1 : "'.($save?'save,':'').'tinyautosave,|,print,|,fullscreen,spellchecker",
			theme_advanced_buttons2:"",
			theme_advanced_button3:"",
			theme_advanced_buttons4:"",
			theme_advanced_toolbar_location : "top",
			theme_advanced_toolbar_align : "left",
			theme_advanced_statusbar_location : "bottom",
			theme_advanced_resizing : true
		';
	}
	/**
	 * Gets the configuration parameters for an advanced editor
	 * @param bool $save Turn the save plugin on or off
	 * @return string
	 */
	static private function getConfigAdvanced($save) {
		global $t;
		return '
			language: "'.$t->get_language().'",
			theme : "advanced",
			skin : "o2k7",
			skin_variant : "black",
			plugins : "safari,spellchecker,pagebreak,style,layer,table,save,tinyautosave,advhr,advimage,advlink,emotions,iespell,inlinepopups,insertdatetime,preview,media,searchreplace,print,contextmenu,paste,directionality,fullscreen,noneditable,visualchars,nonbreaking,xhtmlxtras,template",
			theme_advanced_buttons1 : "'.($save?'save,':'').'tinyautosave,|,bold,italic,underline,strikethrough,|,justifyleft,justifycenter,justifyright,justifyfull,|,styleselect,formatselect,fontselect,fontsizeselect",
			theme_advanced_buttons2 : "cut,copy,paste,pastetext,pasteword,|,search,replace,|,bullist,numlist,|,outdent,indent,blockquote,|,undo,redo,|,link,unlink,anchor,image,cleanup,help,code,|,insertdate,inserttime,preview,|,forecolor,backcolor",
			theme_advanced_buttons3 : "tablecontrols,|,hr,removeformat,visualaid,|,sub,sup,|,charmap,iespell,media,advhr,|,print,|,fullscreen",
			theme_advanced_buttons4 : "styleprops,spellchecker,|,cite,abbr,acronym,del,ins,attribs,|,visualchars,nonbreaking",
			theme_advanced_toolbar_location : "top",
			theme_advanced_toolbar_align : "left",
			theme_advanced_statusbar_location : "bottom",
			theme_advanced_resizing : true
		';
	}
	/**
	 * Outputs the code to create a simple tinymce editor
	 * @param string $selector A tinymce selector for the element to use
	 * @param bool $save Turn the save plugin on or off
	 */
	static function spawnSimple($selector=self::defaultSelector,$save=true) {
		echo '<script language="JavaScript" src="'.self::fetchScriptLocation(self::tinymce).'"></script>
		<script>'.self::fetchScript(self::tinymce,self::simple,$selector,$save).'</script>';
	}
	/**
	 * Outputs the code to create an advanced tinymce editor
	 * @param string $selector A tinymce selector for the element to use
	 * @param bool $save Turn the save plugin on or off
	 */
	static function spawnAdvanced($selector=self::defaultSelector,$save=true) {
		echo '<script language="JavaScript" src="'.self::fetchScriptLocation(self::tinymce).'"></script>
		<script>'.self::fetchScript(self::tinymce,self::advanced,$selector,$save).'</script>';
	}
	/**
	 * Outputs the code to create a simple jquery tinymce editor
	 * @param string $selector A jquery selector for the element to use
	 * @param bool $save Turn the save plugin on or off
	 */
	static function spawnJQuerySimple($selector=self::defaultSelector,$save=true) {
		echo '<script language="JavaScript" src="'.self::fetchScriptLocation(self::jquery).'"></script>
		<script>'.self::fetchScript(self::jquery,self::simple,$selector,$save).'</script>';
	}
	/**
	 * Outputs the code to create an advanced jquery tinymce editor
	 * @param string $selector A jquery selector for the element to use
	 * @param bool $save Turn the save plugin on or off
	 */
	static function spawnJQueryAdvanced($selector=self::defaultSelector,$save=true) {
		echo '<script language="JavaScript" src="'.self::fetchScriptLocation(self::jquery).'"></script>
		<script>'.self::fetchScript(self::jquery,self::advanced,$selector,$save).'</script>';
	}
	/**
	 * Creates the code for the different types of TinyMCE editors
	 * @param int $type Create a tinymce (0) or jquery (1) editor
	 * @param int $mode Create an advanced (0) or simple (1) editor
	 * @param string $selector tinymce or jquery selector for the editor
	 * @param bool $save Turn save plugin on or off
	 * @return string The JavaScript code to insert an editor
	 */
	static function fetchScript($type=self::tinymce,$mode=self::advanced,$selector=self::defaultSelector,$save=true) {
		if($type==self::tinymce) {
			$base='tinyMCE.init({mode:"%2$s", %1$s});';
		}
		elseif($type==self::jquery) {
			$base='$("%2$s").tinymce({script_url: "'.self::fetchScriptLocation().'",%1$s});';
		}
		if($mode==self::advanced) {
			$config=self::getConfigAdvanced($save);
		}
		elseif($mode==self::simple) {
			$config=self::getConfigSimple($save);
		}
		if($selector==self::defaultSelector) {
			if($type==self::tinymce) {
				$selector='textareas';
			}
			elseif($type==self::jquery) {
				$selector='textarea';
			}
		}
		return sprintf($base,$config,$selector);
	}
	/**
	 * Return the script file to include before executing the javascript to enable a tinymce editor
	 * @param int $type Create a tinymce (0) or jquery (1) editor
	 * @return string The location of the script
	 */
	static function fetchScriptLocation($type=self::tinymce) {
		if($type==self::tinymce) {
			return URL_TINYMCE.'/tiny_mce/tiny_mce.js';
		}
		elseif($type==self::jquery) {
			return URL_TINYMCE.'/tiny_mce/jquery.tinymce.js';
		}
	}
}