<?php //@./system_remote/plugins/interfaces.php ?><?php
/**
 * This file defines interfaces for all plugins
 * @package plugins
 */
/**
 * This interface is an entry point for plugins that use a single table
 */
interface pluginSingleTable {
	/**
	 * Get all entries in the table
	 * @return array Nested arrays. The 2nd level array keys use the fields from the table.
	 * @static
	 * @see get_where
	 */
	static function gets();
		/* Example return:
			array(
				array(
					[id]=>1,
					[title]=>"Foo",
					...
				),
				array(
					[id]=>2,
					[title]=>"Bar",
					...
				),
				...
			)
		* The 2nd level array keys use the fields from the table.
		*/
	/**
	 * Gets entry array from the table
	 * @return array The array keys use the fields from the table.
	 * @static
	 * @param int $id The entry id which for the array should be returned
	 */
	static function get($id);
		/* Example return:
			array(
				[id]=>3,
				[title]=>"FooBar",
				...
			)
		* The array keys use the fields from the table.
		*/
	/**
	 * Deleted an entry from the table
	 * @return bool true on success, false on failure
	 * @static
	 * @param int $id The entry id which should be deleted
	 */
	static function delete($id);
	/**
	 * Add an entry to the table
	 * @return int Inserted entry id
	 * @static
	 * @param array $properties array in which the values to insert into the tables have been defined.
	 */
	static function add($properties);
	/**
	 * Change the properties of an entry
	 * @return bool true on success, false on failure
	 * @static
	 * @param int $id The id of the entry to be edited
	 * @param array $properties The new properties for the entry. Use the same names as the table fields
	 */
	static function edit($id,$properties);
	/**
	 * Get the entries for which the defined properties apply
	 * @return array Nested arrays. The 2nd level array keys use the fields from the table.
	 * @static
	 * @param array $properties array in which the values the entries should match are given. Use table field names
	 * @see gets
	 */
	static function get_where($properties);
}
/**
 * This interface is an entrypoint for plugins that use a single table and are moveable
 */
interface pluginSingleTableMoveable extends pluginSingleTable {
	/**
	 * Moves an entry up or down
	 * @return bool	 true on success, false on failure
	 * @static
	 * @param int $id The id of the entry to be moved
	 * @param int $where Which way the entry to move to. If it is >0, the entry will be moved up, if it is <0, the entry will be moved down.
	 */
	static function move($id,$where);
}